import os
import shutil

from cpl_core.console import Console
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_query.extension import List

from bot_data.abc.migration_abc import MigrationABC
from bot_data.model.migration_history import MigrationHistory
from migration_to_sql.mock.mock_db_context import MockDBContext


class MigrationService:
    def __init__(
        self,
        services: ServiceProviderABC,
        db: MockDBContext,
    ):
        self._services = services

        self._db = db
        self._cursor = db.cursor

        # self._migrations: List[MigrationABC] = (
        #     List(type, MigrationABC.__subclasses__()).order_by(lambda x: x.name.split("_")[0]).then_by(lambda x: x.prio)
        # )

    def migrate(self):
        path = f"../../src/bot_data/scripts"
        if not os.path.exists(path):
            os.makedirs(path)
        else:
            shutil.rmtree(path)
            os.makedirs(path)

        for migration in self._services.get_services(MigrationABC):
            migration_id = type(migration).__name__
            try:
                # migration_as_service: MigrationABC = self._services.get_service(migration)
                # save upgrade scripts
                self._db.set_migration(migration.name, True)
                migration.upgrade()
                self._cursor.execute(MigrationHistory(migration_id).insert_string)
                self._db.save_changes()

                # save downgrade scripts
                self._db.set_migration(migration.name)
                migration.downgrade()
                self._cursor.execute(MigrationHistory(migration_id).insert_string)
                self._db.save_changes()

            except Exception as e:
                Console.error(e)
