import os
import textwrap
from typing import Optional

from cpl_core.database import DatabaseSettings
from cpl_core.database.context import DatabaseContextABC

from migration_to_sql.mock.mock_cursor import MockCursor


class MockDBContext(DatabaseContextABC):
    def __init__(self):
        DatabaseContextABC.__init__(self)
        self._migration_version: Optional[str] = None
        self._migration_name: Optional[str] = None
        self._migration_script: Optional[str] = None

        self._old_version: Optional[str] = None
        self._old_name: Optional[str] = None

        self._index: int = 0

    @property
    def cursor(self) -> MockCursor:
        cursor = MockCursor()
        cursor.execute = self.execute
        return cursor

    def set_migration(self, name: str, upgrade: bool = False):
        self._migration_name = f'{name.split("_")[1].replace("Migration", "")}_{"up" if upgrade else "down"}'
        self._migration_version = name.split("_")[0]
        self._migration_script = ""

    def connect(self, database_settings: DatabaseSettings):
        pass

    def select(self, statement: str) -> list[tuple]:
        pass

    def execute(self, query: str, *args, **kwargs):
        if "MigrationHistory" in query:
            return None
        else:
            self._migration_script += f"{textwrap.dedent(query)}\n\n"

    def save_changes(self):
        path = f"../../src/bot_data/scripts/{self._migration_version}"
        if not os.path.exists(path):
            os.makedirs(path)

        if (
            self._old_name is not None
            and self._migration_name is not None
            and self._old_name.split("_")[0] == self._migration_name.split("_")[0]
        ):
            pass
        elif self._old_version == self._migration_version:
            self._index += 1
        else:
            self._index = 1

        script = textwrap.dedent(self._migration_script)
        with open(f"{path}/{self._index}_{self._migration_name}.sql", "w+") as f:
            f.write(script)
            f.close()

        self._old_version = self._migration_version
        self._old_name = self._migration_name

        self._migration_name = None
        self._migration_version = None
        self._migration_script = None
